/************************************************************************
Project(s):	            Infrastructure Growth Meta Analysis
Do File: 			    mainresults.do
Initial file by : 		Stephane Straub and Dhruv Jain
This version : 			Dhruv Jain
Created version: 		13/12/2023
Updated version: 	    
This version :			20/06/2024
Stata Version :			18
********************************************************************************
Description: This do file prepares the following outputs
*********************************************************************************/

clear all
set matsize 1500
set more off

*ssc install outreg2
*ssc install lassopack


cd "$project"

use "data/mradata.dta", clear

*** create some aditional variables ***
gen se_1 = 1/se
gen se2= se*se	
gen z=0
gen y=1.96/finalcoeff if finalcoeff>0
replace y= -1.96/finalcoeff if finalcoeff<0

gen endocare = 1 - endogeneity
label var endocare "Dummy=1 if endogeneity addressed"

drop if tstat > 300

// Redefine sectors

//Create a separate dummy variable for public capital:

gen pubcap = 0
replace pubcap=1 if strpos(indepvar, "capital") 
replace pubcap=1 if strpos(indepvar, "Capital") 
replace pubcap=1 if strpos(indepvar, "investment") 
replace pubcap=1 if strpos(indepvar, "Investment") 
replace pubcap=1 if strpos(indepvar, "GFCF") 
replace pubcap=1 if strpos(indepvar, "govt. spending") 
replace pubcap=1 if strpos(indepvar, "Govt. spending") 
tab pubcap

//Sector classification: Digital (1), Energy (2), Transport (3), Cross-sectoral (4)

replace infra=4 if infra==0
tab infra


********************************************************************************
**************************** F I G U R E S *************************************
********************************************************************************



*** Figure 2 ***

twoway (scatter se_1 finalcoeff if finalcoeff > -3  & finalcoeff < 3 & se_1< 200, msize(vsmall) mfc(none) mlc(blue%30)) (line se_1 z if finalcoeff > -3  & finalcoeff < 3 & se_1< 220, lwidth(medium) lcolor(red)),  ytitle(1/Standard Error) xtitle(Coefficient) legend(off) || line y finalcoeff if finalcoeff > -3  & finalcoeff<0 & y<220,  lwidth(medium) lcolor(black) lpattern(dash) sort || line y finalcoeff if finalcoeff>0  & finalcoeff < 3 & y<220,  lwidth(medium) lpattern(dash) lcolor(black) sort scheme(s1mono) plotregion(lwidth(none))

graph export "results/fig2.png", replace



********************************************************************************



*** Figure 3 ***

gen t=abs(coeffpaper/se) 
gen pv=2*(1-normal(abs(t)))
order pv, after(t)


// panel A

capture drop density1 xaxis1 ci1_1 ci1_2
kdens t if t <= 10, ll(0) ul(10) gen(density1 xaxis1) ci(ci1_1 ci1_2) nograph
twoway (hist t if t <= 10,  fcolor(gray%50) lcolor(blue%10) width(0.1) barwidth(0.09) ) ///
(line density1 xaxis1,lcolor(black%90) lpattern(longdash) lwidth(medthick)), ///
legend(off) graphregion(color(white)) ylabel(, nogrid) title("", color(black)) xtitle("z-statistic") ytitle("Density") ///
xline(1.65 1.96 2.58, lwidth(medium) lcolor(maroon)) xlabel(0 1 1.65 "*" 1.96 "**" 2.58 "***" 3 4 5 6 7 8 9 10) scheme(s1mono)

graph export "results/fig3a.png", replace


// panel B

twoway (hist pv if pv <= .15 & pv>.0025, fcolor(lavender%40) lcolor(black%70) width(0.0025) barwidth(0.0025) ), ///
legend(off) graphregion(color(white)) ylabel(, nogrid) title("", color(black)) xtitle("p-value") ytitle("Density") ///
xline(.1 .05 .01, lwidth(medium) lcolor(maroon)) xlabel(0.01 .05 .1 .15 ) scheme(s1mono)

graph export "results/fig3b.png", replace



********************************************************************************



*** Figure 4 ***


// panel A

preserve

use "data/pessefig_1.dta", clear

local group1 sector
local group2 sample


sort `group1' `group2'
gen x = _n
replace x = _n + 2 if _n >= 3
replace x = _n + 4 if _n >= 5
replace x = _n + 6 if _n >= 7

replace x=x+1

gen yu = coef + 1.96*se
gen yl = coef - 1.96*se

separate coef, by(`group2')
separate yu, by(`group2')
separate yl, by(`group2')


twoway (scatter coef1 x , msize(medium) mcolor(darkgray) mlabcolor(darkgray)) ///
       (rcap yu1 yl1 x, lcolor(gray) lwidth(medthick))  ///
       (scatter coef2 x , msize(medium) mcolor(blue) mlabcolor(blue) msymbol(T)) ///
       (rcap yu2 yl2 x, lcolor(navy) lwidth(medthick)),  ///
	   xlabel(none) xtitle(" ") yscale(range(0 0.23)) ///
	   ylabel(,nogrid) ///
	   text(-0.015 1.5 "Cross-Sectoral", place(e)) ///
	   text(-0.015 6 "Digital", place(e)) ///
	   text(-0.015 10 "Energy", place(e)) ///
	   text(-0.015 13.5 "Transport", place(e)) ///
	   graphregion(fcolor(white) ifcolor(white) ilcolor(white)) ///
	   legend(order(1 "Developed" 3 "Developing") position(1) row(1) region(style(none)) ring(0)) scheme(s2color)
	   
	   
graph export "results/fig4a.png", replace 

restore	


// panel B


preserve

use "data/pessefig_2.dta", clear 

drop if coef==.

drop if _n==8 | _n==10 | _n==11

encode sector, gen(y)

local group1 sector
local group2 sample

sort `group1' `group2'
gen x = _n

replace x = _n + 2 if _n >= 5
replace x = _n + 3 if _n >= 7

*drop if _n==3 | _n==4 | _n==11 | _n==12 // dropping large estimates

replace x = x+0.5 if x >=6
replace x = x+0.5 if x >=9

gen n=_n
order n

replace x=x-0.5 if !mod(n,2)

replace x=x-0.75 if n==1 | n==2 
replace x=x+2 if n>4
replace x=x+2 if n>6
replace x=x+1 if n>8

gen yu = coef + 1.96*se
gen yl = coef - 1.96*se

local group1 sector
local group2 sample
separate coef, by(`group2')
separate yu, by(`group2')
separate yl, by(`group2')


twoway (scatter coef1 x , msize(medium) mcolor(darkgray) mlabcolor(gray)) ///
       (rcap yu1 yl1 x, lcolor(gray) lwidth(med))  ///
       (scatter coef2 x , msize(medium) mcolor(blue) mlabcolor(blue) msymbol(T)) ///
       (rcap yu2 yl2 x, lcolor(navy) lwidth(med)),  ///
	   xlabel(none) xtitle(" ") ylabel(, nogrid) xscale(r(-0.5 20)) yscale(r(0 0.22)) ///
	   text(-0.02 1 "Digital", place(e)) ///
	   text(-0.01 -1.5 "Output Macro", place(e) size(small)) ///
	   text(-0.01 1.8 "Output Micro", place(e) size(small)) ///
	   text(-0.02 8.5 "Energy", place(e)) ///
	   text(-0.01 8 "Output Macro", place(e) size(small)) ///
	   text(-0.02 15.5 "Transport", place(e)) ///
	   text(-0.01 14.5 "Labor", place(e) size(small)) ///
	   text(-0.01 16.5 "Output Macro", place(e) size(small)) ///
	   graphregion(fcolor(white) ifcolor(white) ilcolor(white)) ///
	   legend(order(1 "Developed" 3 "Developing") position(1) row(1) region(style(none)) ring(0)) scheme(s2color)

graph export "results/fig4b.png", replace 

restore



********************************************************************************
****************************** T A B L E S *************************************
********************************************************************************




*** Table 1 ***

tab depvarcat infra // note some categories have been combined for easier presentation



*** Table 2 ***

tab link infra, column



*** Table 3 ***


regress tstat se_1, cluster(id) robust  // Whole sample
outreg2 using "results/table3.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (FAT-PET estimates) sdec(3) replace


regress tstat se_1 if infra == 1, cluster(id) robust  // Digital
outreg2 using "results/table3.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (FAT-PET estimates) sdec(3) append


regress tstat se_1 if infra == 2, cluster(id) robust // Energy 
outreg2 using "results/table3.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (FAT-PET estimates) sdec(3) append


regress tstat se_1 if infra == 3, cluster(id) robust  // Transport
outreg2 using "results/table3.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (FAT-PET estimates) sdec(3) append


regress tstat se_1 if infra == 4, cluster(id) robust // Cross
outreg2 using "results/table3.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (FAT-PET estimates) sdec(3) append



*** Table 4 ***


regress tstat se_1 se, cluster(id) robust noconstant // Whole sample
outreg2 using "results/table4.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) keep (se_1 se) sdec(3) replace


regress tstat se_1 se if infra == 1, cluster(id) robust noconstant // Digital
outreg2 using "results/table4.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) keep (se_1 se) sdec(3) append


regress tstat se_1 se if infra == 2, cluster(id) robust noconstant // Energy
outreg2 using "results/table4.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) keep (se_1 se) sdec(3) append


regress tstat se_1 se if infra == 3, cluster(id) robust noconstant // Transport
outreg2 using "results/table4.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) keep (se_1 se) sdec(3) append


regress tstat se_1 se if infra == 4, cluster(id) robust noconstant // Cross
outreg2 using "results/table4.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) keep (se_1 se) sdec(3) append




********************************************************************************


*** Table 5 (and appendix A8 to A11) ***


// Note 1: Some additional variables need to be created for table 5
// Note 2: Main effect reported in the table is given by the coefficient on the constant term in the following regressions 
// Note 3: Table 5 only reports the constant term as the main effect and Appendix tables A8, A9, A10 and A11 report the coefficients of the full models.


local vars "publication academic govt international pubyear crosscountry developing time_med regiondata districtdata firmdata hhddata pf spatial iv did fe coint pubcap endocare "

foreach var of local vars {
	 summarize `var', meanonly
	 gen `var'_c = `var' - r(mean)
			 }

global x *_c //moderators


// regression output program

program drop _all
program define myreg
outreg2 using "results/table5.xls", append stats(coef se) excel addtext(Model Selection, "`1'", Sample, "`2'")
end




// Panel A: without moderators

reg finalcoeff se2 [aweight=se^(-2)], cluster(id) robust 
myreg "No Mod" "Full"

reg finalcoeff se2 if infra==1 [aweight=se^(-2)], cluster(id) robust 
myreg "No Mod" "Digital"

reg finalcoeff se2 if infra==2 [aweight=se^(-2)], cluster(id) robust 
myreg "No Mod" "Energy"

reg finalcoeff se2 if infra==3 [aweight=se^(-2)], cluster(id) robust 
myreg "No Mod" "Transport"

reg finalcoeff se2 if infra==4 [aweight=se^(-2)], cluster(id) robust 
myreg "No Mod" "Cross-Sector"


// Panel B: regularized lasso

rlasso finalcoeff se2 $x [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' [aweight=se^(-2)], cluster(id) robust
myreg "RLASSO" "Full"

rlasso finalcoeff se2 $x if infra==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //digital
reg finalcoeff se2 `e(selected)' if infra==1 [aweight=se^(-2)], cluster(id) robust
myreg "RLASSO" "Digital"

rlasso finalcoeff se2 $x if infra==2 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //energy
reg finalcoeff se2 `e(selected)' if infra==2 [aweight=se^(-2)], cluster(id) robust
myreg "RLASSO" "Energy"

rlasso finalcoeff se2 $x if infra==3 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //transport
reg finalcoeff se2 `e(selected)' if infra==3 [aweight=se^(-2)], cluster(id) robust
myreg "RLASSO" "Transport"

rlasso finalcoeff se2 $x if infra==4 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //cross-sector
reg finalcoeff se2 `e(selected)' if infra==4 [aweight=se^(-2)], cluster(id) robust
myreg "RLASSO" "Cross-Sector"



// Panel C: cross-validation lasso

/* notes: 1. We could not figure out a way to directly get the list of chosen predictors after running cvlasso, so we add them to our post-OLS manually
2. cvlasso command also does not allow weights and clustering 
3. IMPORTANT: cvlasso selects different covariates for the same regression if you run it multiple times so the model may differ during replication */


cvlasso finalcoeff se2 $x, notpen(se2) lopt ols
global chosen publication_c pubyear_c crosscountry_c developing_c regiondata_c ///
districtdata_c spatial_c iv_c fe_c
reg finalcoeff se2 $chosen [aweight=se^(-2)], cluster(id) robust
myreg "CVLASSO" "Full"

cvlasso finalcoeff se2 $x if infra==1, notpen(se2) lopt ols //digital
global chosen firmdata_c
reg finalcoeff se2 $chosen if infra==1 [aweight=se^(-2)], cluster(id) robust
myreg "CVLASSO" "Digital"

cvlasso finalcoeff se2 $x if infra==2, notpen(se2) lopt ols //energy
global chosen districtdata_c spatial_c iv_c
reg finalcoeff se2 $chosen if infra==2 [aweight=se^(-2)], cluster(id) robust
myreg "CVLASSO" "Energy"

cvlasso finalcoeff se2 $x if infra==3, notpen(se2) lopt ols //transport
global chosen publication_c international_c crosscountry_c developing_c time_med_c ///
districtdata_c hhddata_c spatial_c did_c fe_c coint_c pubcap_c
reg finalcoeff se2 $chosen if infra==3 [aweight=se^(-2)], cluster(id) robust
myreg "CVLASSO" "Transport"

cvlasso finalcoeff se2 $x if infra==4, notpen(se2) lopt ols //cross-sector
global chosen publication_c academic_c pubyear_c crosscountry_c time_med_c regiondata_c districtdata_c firmdata_c pf_c iv_c fe_c pubcap_c endocare_c
reg finalcoeff se2 $chosen if infra==4 [aweight=se^(-2)], cluster(id) robust
myreg "CVLASSO" "Cross-Sector"


// Panel D and E: Bayesian Model Averaging 

/* notes: 
1. we first perform BMA separately for different sectors and then we run the regressions using the results of BMA
2. we could not find a way to directly add the variables with (i) either PIP>0.1 or (ii) variables in the model with the highest PMP, directly in the OLS regression we run subsequently. So we add them manually in our regressions 
3. BMA available only with Stata 18 */


*... full sample
					
bmaregress finalcoeff (se2, always) publication_c academic_c govt_c ///
international_c pubyear_c crosscountry_c developing_c time_med_c ///
regiondata_c districtdata_c firmdata_c hhddata_c pf_c spatial_c iv_c did_c ///
fe_c coint_c pubcap_c endocare_c, enumeration
bmaregress, saving(bmareg0, replace) 
estimates store bmareg0
*estimates restore bmareg0 // command to restore this model and review it
bmastats models, cumulative // command to check for model with highest PMP

*... digital

bmaregress finalcoeff (se2, always) publication_c academic_c govt_c ///
international_c pubyear_c crosscountry_c developing_c time_med_c ///
regiondata_c districtdata_c firmdata_c hhddata_c pf_c spatial_c iv_c did_c ///
fe_c coint_c pubcap_c endocare_c if infra==1, enumeration
bmaregress, saving(bmareg1, replace)
estimates store bmareg1
bmastats models, cumulative // command to check for model with highest PMP


*... energy

bmaregress finalcoeff (se2, always) publication_c academic_c govt_c ///
international_c pubyear_c crosscountry_c developing_c time_med_c ///
regiondata_c districtdata_c firmdata_c hhddata_c pf_c spatial_c iv_c did_c ///
fe_c coint_c pubcap_c endocare_c if infra==2, enumeration
bmaregress, saving(bmareg2, replace)
estimates store bmareg2
bmastats models, cumulative // command to check for model with highest PMP


*...transport

bmaregress finalcoeff (se2, always) publication_c academic_c govt_c ///
international_c pubyear_c crosscountry_c developing_c time_med_c ///
regiondata_c districtdata_c firmdata_c hhddata_c pf_c spatial_c iv_c did_c ///
fe_c coint_c pubcap_c endocare_c if infra==3, enumeration
bmaregress, saving(bmareg3, replace)
estimates store bmareg3
bmastats models, cumulative // command to check for model with highest PMP


*...cross-sector

bmaregress finalcoeff (se2, always) publication_c academic_c govt_c ///
international_c pubyear_c crosscountry_c developing_c time_med_c ///
regiondata_c districtdata_c firmdata_c hhddata_c pf_c spatial_c iv_c did_c ///
fe_c coint_c pubcap_c endocare_c if infra==4, enumeration
bmaregress, saving(bmareg4, replace)
estimates store bmareg4
bmastats models, cumulative // command to check for model with highest PMP


// panel D:  OLS regressions with controls having PIP>0.10

reg finalcoeff se2 spatial_c iv_c crosscountry_c [aweight=se^(-2)], cluster(id) robust 
myreg "BMA PIP>0.1" "Full"

reg finalcoeff se2 firmdata_c if infra==1 [aweight=se^(-2)], cluster(id) robust 
myreg "BMA PIP>0.1" "Digital"

reg finalcoeff se2 districtdata_c spatial_c iv_c time_med_c if infra==2 [aweight=se^(-2)], cluster(id) robust 
myreg "BMA PIP>0.1" "Energy"

reg finalcoeff se2 crosscountry_c fe_c international_c spatial_c developing_c time_med_c districtdata_c if infra==3 [aweight=se^(-2)], cluster(id) robust 
myreg "BMA PIP>0.1" "Transport"

reg finalcoeff se2 crosscountry_c publication_c pf_c regiondata_c districtdata_c firmdata_c iv_c endocare_c pubcap_c pubyear_c if infra==4 [aweight=se^(-2)], cluster(id) robust 
myreg "BMA PIP>0.1" "Cross-Sector"


// panel E:  OLS regressions with controls of the model with highest PMP

reg finalcoeff se2 spatial_c [aweight=se^(-2)], cluster(id) robust //(PMP=0.5695)
myreg "BMA Top" "Full"

reg finalcoeff se2 firmdata_c if infra==1 [aweight=se^(-2)], cluster(id) robust //(PMP=0.8059)
myreg "BMA Top" "Digital"

reg finalcoeff se2 spatial_c if infra==2 [aweight=se^(-2)], cluster(id) robust //(PMP=0.281)
myreg "BMA Top" "Energy"

reg finalcoeff se2 international_c crosscountry_c spatial_c fe_c if infra==3 [aweight=se^(-2)], cluster(id) robust 
//(PMP=.2391)
myreg "BMA Top" "Transport"

reg finalcoeff se2 publication_c crosscountry_c regiondata_c districtdata_c firmdata_c pf_c if infra==4 [aweight=se^(-2)], cluster(id) robust 
//(PMP=.2117)
myreg "BMA Top" "Cross-Sector"



********************************************************************************



*** Table 6 ***


// Note: Main effect reported in the table is given by the coefficient on the constant term in the following regressions 

	
// Panel A: 
			
// Digital x output micro  
reg finalcoeff se2 if infra == 1 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) replace addtext(Sample, "Digital-micro")

// Digital x output macro 
reg finalcoeff se2 if infra == 1 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Digital-macro")

// Energy x output micro 
reg finalcoeff se2 if infra == 2 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Energy-micro")

// Energy x output macro 
reg finalcoeff se2 if infra == 2 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Energy-macro")

// Energy x labor
reg finalcoeff se2 if infra == 2 & depvarcatcode==3 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Energy-labor")

// Transport x output micro 
reg finalcoeff se2 if infra == 3 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Transport-micro")

// Transport x output macro 
reg finalcoeff se2 if infra == 3 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Transport-macro")

// Transport x labor
reg finalcoeff se2 if infra == 3 & depvarcatcode==3 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Transport-labor")

// Transport x population
reg finalcoeff se2 if infra == 3 & depvarcatcode==7 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Transport-population")

// Transport x trade
reg finalcoeff se2 if infra == 3 & depvarcatcode==5 [aweight=se^(-2)], cluster(id) robust 
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) append addtext(Sample, "Transport-trade")



// Panel B:

// Digital x output micro 
rlasso finalcoeff se2 $x if infra == 1 & depvarcatcode==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 1 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Digital-micro")
		
// Digital x output macro 
rlasso finalcoeff se2 $x if infra == 1 & depvarcatcode==2 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 1 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Digital-macro")

// Energy x output micro 
rlasso finalcoeff se2 $x if infra == 2 & depvarcatcode==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 2 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Energy-micro")

// Energy x output macro 
rlasso finalcoeff se2 $x if infra == 2 & depvarcatcode==2 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 2 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Energy-macro")

// Energy x labor
rlasso finalcoeff se2 $x if infra == 2 & depvarcatcode==3 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 2 & depvarcatcode==3 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Energy-labor")

// Transport x output micro 
rlasso finalcoeff se2 $x if infra == 3 & depvarcatcode==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 3 & depvarcatcode==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Transport-micro")

// Transport x output macro 
rlasso finalcoeff se2 $x if infra == 3 & depvarcatcode==2 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 3 & depvarcatcode==2 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Transport-macro")

// Transport x labor
rlasso finalcoeff se2 $x if infra == 3 & depvarcatcode==3 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 3 & depvarcatcode==3 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Transport-labor")

// Transport x population
rlasso finalcoeff se2 $x if infra == 3 & depvarcatcode==7 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 3 & depvarcatcode==7 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Transport-population")

// Transport x trade
rlasso finalcoeff se2 $x if infra == 3 & depvarcatcode==5 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) 
reg finalcoeff se2 `e(selected)' if infra == 3 & depvarcatcode==5 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table6.xls", alpha(0.01, 0.05, 0.1) symbol(***, **, *) bdec(4) title (PEESE estimates) sdec(3) append addtext(Sample, "Transport-trade")



********************************************************************************



*** Table 7 ***


// Note: Main effect reported in the table is given by the coefficient on the constant term in the following regressions 



// Panel A: Exclude outliers

gen outlier=0
replace outlier=1 if se_1>=200 & abs(tstat)>4

rlasso finalcoeff se2 $x if outlier==0 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) // full sample
reg finalcoeff se2 `e(selected)' if outlier==0  [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Out:Full")

rlasso finalcoeff se2 $x if infra==1 & outlier==0 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //digital
reg finalcoeff se2 `e(selected)' if infra==1 & outlier==0 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Out:Digital")

rlasso finalcoeff se2 $x if infra==2 & outlier==0 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //energy
reg finalcoeff se2 `e(selected)' if infra==2 & outlier==0 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Out:Energy")

rlasso finalcoeff se2 $x if infra==3 & outlier==0 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //transport
reg finalcoeff se2 `e(selected)' if infra==3 & outlier==0 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Out:Transport")

rlasso finalcoeff se2 $x if infra==4 & outlier==0 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //cross-sector
reg finalcoeff se2 `e(selected)' if infra==4 & outlier==0 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Out:Cross-Sec")


// Panel B: elasticity reported in original study

rlasso finalcoeff se2 $x if elasticity==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) // full sample
reg finalcoeff se2 `e(selected)' if elasticity==1  [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Elas:Full")

rlasso finalcoeff se2 $x if infra==1 & elasticity==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //digital
reg finalcoeff se2 `e(selected)' if infra==1 & elasticity==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Elas:Digital")

rlasso finalcoeff se2 $x if infra==2 & elasticity==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //energy
reg finalcoeff se2 `e(selected)' if infra==2 & elasticity==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Elas:Energy")

rlasso finalcoeff se2 $x if infra==3 & elasticity==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //transport
reg finalcoeff se2 `e(selected)' if infra==3 & elasticity==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Elas:Transport")

rlasso finalcoeff se2 $x if infra==4 & elasticity==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //cross-sector
reg finalcoeff se2 `e(selected)' if infra==4 & elasticity==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Elas:Cross-Sec")


// Panel C: endogeneity addressed

rlasso finalcoeff se2 $x if endocare==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) // full sample
reg finalcoeff se2 `e(selected)' if endocare==1  [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Endoc:Full")

rlasso finalcoeff se2 $x if infra==1 & endocare==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //digital
reg finalcoeff se2 `e(selected)' if infra==1 & endocare==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Endoc:Digital")

rlasso finalcoeff se2 $x if infra==2 & endocare==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //energy
reg finalcoeff se2 `e(selected)' if infra==2 & endocare==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Endoc:Energy")

rlasso finalcoeff se2 $x if infra==3 & endocare==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //transport
reg finalcoeff se2 `e(selected)' if infra==3 & endocare==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Endoc:Transport")

rlasso finalcoeff se2 $x if infra==4 & endocare==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //cross-sector
reg finalcoeff se2 `e(selected)' if infra==4 & endocare==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "Endoc:Cross-Sec")

// Panel D: single-country estimates

rlasso finalcoeff se2 $x if countries==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) // full sample
reg finalcoeff se2 `e(selected)' if countries==1  [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "SC:Full")

rlasso finalcoeff se2 $x if infra==1 & countries==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //digital
reg finalcoeff se2 `e(selected)' if infra==1 & countries==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "SC:Digital")

rlasso finalcoeff se2 $x if infra==2 & countries==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //energy
reg finalcoeff se2 `e(selected)' if infra==2 & countries==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "SC:Energy")

rlasso finalcoeff se2 $x if infra==3 & countries==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //transport
reg finalcoeff se2 `e(selected)' if infra==3 & countries==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "SC:Transport")

rlasso finalcoeff se2 $x if infra==4 & countries==1 [aweight=se^(-2)], pnotpen(se2) robust cluster(id) maxpsiiter(5) //cross-sector
reg finalcoeff se2 `e(selected)' if infra==4 & countries==1 [aweight=se^(-2)], cluster(id) robust
outreg2 using "results/table7.xls", append stats(coef se) excel addtext(Sample, "SC:Cross-Sec")
